#!/bin/bash
# -----------------------------------------------------------------------------------------
#  Example Installation Script Template
#  
#  This convenience script encapsulates command-line instructions highlighted in
#  an OpenHPC Install Guide that can be used as a starting point to perform a local
#  cluster install beginning with bare-metal. Necessary inputs that describe local
#  hardware characteristics, desired network settings, and other customizations
#  are controlled via a companion input file that is used to initialize variables 
#  within this script.
#   
#  Please see the OpenHPC Install Guide(s) for more information regarding the
#  procedure. Note that the section numbering included in this script refers to
#  corresponding sections from the companion install guide.
# -----------------------------------------------------------------------------------------

inputFile=${OHPC_INPUT_LOCAL:-/opt/ohpc/pub/doc/recipes/centos7/input.local}

if [ ! -e ${inputFile} ];then
   echo "Error: Unable to access local input file -> ${inputFile}"
   exit 1
else
   . ${inputFile} || { echo "Error sourcing ${inputFile}"; exit 1; }
fi

# ---------------------------- Begin OpenHPC Recipe ---------------------------------------
# Commands below are extracted from an OpenHPC install guide recipe and are intended for 
# execution on the master SMS host.
# -----------------------------------------------------------------------------------------

# Disable firewall 
systemctl disable firewalld
systemctl stop firewalld


# --------------------------------------
# Enable xCAT repositories (Section 3.1)
# --------------------------------------
yum -y install yum-utils
wget -P /etc/yum.repos.d https://xcat.org/files/xcat/repos/yum/latest/xcat-core/xcat-core.repo
wget -P /etc/yum.repos.d https://xcat.org/files/xcat/repos/yum/xcat-dep/rh7/x86_64/xcat-dep.repo

# ------------------------------------------------------------
# Add baseline OpenHPC and provisioning services (Section 3.2)
# ------------------------------------------------------------
yum -y install xCAT
. /etc/profile.d/xcat.sh


# -------------------------------------------------------
# Complete basic xCAT setup for master node (Section 3.3)
# -------------------------------------------------------
ifconfig ${sms_eth_internal} ${sms_ip} netmask ${internal_netmask} up
chdef -t site dhcpinterfaces="xcatmn|${sms_eth_internal}"

# ------------------------------------------------------
# Initialize OS images for use with xCAT (Section 3.4.1)
# ------------------------------------------------------
copycds ${iso_path}/CentOS-7-x86_64-DVD-1810.iso

# Add hosts to cluster (Section 3.5)
for ((i=0; i<$num_computes; i++)) ; do
   mkdef -t node ${c_name[$i]} groups=compute,all ip=${c_ip[$i]} mac=${c_mac[$i]} netboot=xnba \
   arch=x86_64 bmc=${c_bmc[$i]} bmcusername=${bmc_username} bmcpassword=${bmc_password} \
   mgt=ipmi serialport=0 serialspeed=115200
done
# Set default root password
chtab key=system passwd.username=root passwd.password=`openssl rand -base64 12`

# Define local domainname
chdef -t site domain=${domain_name}

# Setup IPoIB networking
if [[ ${enable_ipoib} -eq 1 ]];then
     chdef -t network -o ib0 mask=$ipoib_netmask net=${c_ipoib[0]}
     chdef compute -p postbootscripts=confignics
     for ((i=0; i<$num_computes; i++)) ; do
        chdef ${c_name[i]} nicips.ib0=${c_ipoib[i]} nictypes.ib0="InfiniBand" nicnetworks.ib0=ib0
     done
fi

# Complete networking setup, associate provisioning image
makehosts
makenetworks
makedhcp -n
makedns -n
nodeset compute osimage=centos7.6-x86_64-install-compute

# ----------------------------------
# Set nodes to netboot (Section 3.6)
# ----------------------------------
rsetboot compute net

# ------------------
# Boot compute nodes
# ------------------
rpower compute reset


# --------------------------------------------------------
# Download OHPC repo and create local mirror (Section 3.1)
# --------------------------------------------------------
wget http://build.openhpc.community/dist/1.3.6/OpenHPC-1.3.6.CentOS_7.x86_64.tar
mkdir -p $ohpc_repo_dir
tar xvf OpenHPC-1.3.6.CentOS_7.x86_64.tar -C $ohpc_repo_dir
$ohpc_repo_dir/make_repo.sh

# Verify OpenHPC repository has been enabled before proceeding

yum repolist | grep -q OpenHPC
if [ $? -ne 0 ];then
   echo "Error: OpenHPC repository must be enabled locally"
   exit 1
fi
 yum -y install epel-release
 yum -y install ohpc-base
# Enable NTP services on SMS host
systemctl enable ntpd.service
echo "server ${ntp_server}" >> /etc/ntp.conf
systemctl restart ntpd

# -------------------------------------------------------------
# Add resource management services on master node (Section 4.4)
# -------------------------------------------------------------
yum -y install ohpc-slurm-server
perl -pi -e "s/ControlMachine=\S+/ControlMachine=${sms_name}/" /etc/slurm/slurm.conf

# ----------------------------------------
# Update node configuration for slurm.conf
# ----------------------------------------
if [[ ${update_slurm_nodeconfig} -eq 1 ]];then
     perl -pi -e "s/^NodeName=.+$/#/" /etc/slurm/slurm.conf
     perl -pi -e "s/ Nodes=c\S+ / Nodes=c[1-$num_computes] /" /etc/slurm/slurm.conf
     echo -e ${slurm_node_config} >> /etc/slurm/slurm.conf
fi

# -----------------------------------------------------------------------
# Optionally add InfiniBand support services on master node (Section 4.5)
# -----------------------------------------------------------------------
if [[ ${enable_ib} -eq 1 ]];then
     yum -y groupinstall "InfiniBand Support"
     yum -y install infinipath-psm
     systemctl start rdma
fi

# Optionally enable opensm subnet manager
if [[ ${enable_opensm} -eq 1 ]];then
     yum -y install opensm
     systemctl enable opensm
     systemctl start opensm
fi

# Optionally enable IPoIB interface on SMS
if [[ ${enable_ipoib} -eq 1 ]];then
     # Enable ib0
     cp /opt/ohpc/pub/examples/network/centos/ifcfg-ib0 /etc/sysconfig/network-scripts
     perl -pi -e "s/master_ipoib/${sms_ipoib}/" /etc/sysconfig/network-scripts/ifcfg-ib0
     perl -pi -e "s/ipoib_netmask/${ipoib_netmask}/" /etc/sysconfig/network-scripts/ifcfg-ib0
     ifup ib0
fi

# ----------------------------------------------------------------------
# Optionally add Omni-Path support services on master node (Section 4.6)
# ----------------------------------------------------------------------
if [[ ${enable_opa} -eq 1 ]];then
     yum -y install opa-basic-tools
     systemctl start rdma
fi

# Optionally enable OPA fabric manager
if [[ ${enable_opafm} -eq 1 ]];then
     yum -y install opa-fm
     systemctl enable opafm
     systemctl start opafm
fi

# --------------------------------------------------------------------
# Setup nodes repositories and Install OHPC components (Section 4.6.1)
# --------------------------------------------------------------------
psh compute yum --setopt=\*.skip_if_unavailable=1 -y install yum-utils
psh compute yum-config-manager --disable CentOS\*
psh compute yum-config-manager --add-repo=http://$sms_ip/$ohpc_repo_dir/OpenHPC.local.repo
psh compute "perl -pi -e 's/file:\/\/\@PATH\@/http:\/\/$sms_ip"${ohpc_repo_dir//\//"\/"}"/s' /etc/yum.repos.d/OpenHPC.local.repo"

# ---------------------------------
# Configure access to EPEL packages
# ---------------------------------
mkdir -p $epel_repo_dir
yum -y install yum-utils createrepo
yumdownloader --destdir $epel_repo_dir fping qstat libconfuse python34 python34-libs
createrepo $epel_repo_dir
psh compute yum-config-manager --add-repo=http://$sms_ip/$epel_repo_dir
psh compute "echo gpgcheck=0 >> /etc/yum.repos.d/$sms_ip"${epel_repo_dir//\//"_"}.repo
psh compute echo -e %_excludedocs 1 \>\> ~/.rpmmacros

# --------------------------------------------
# Add OpenHPC base components to compute image
# --------------------------------------------
psh compute yum -y install ohpc-base-compute
# Add OpenHPC components to compute instance
psh compute yum -y install ohpc-slurm-client
psh compute yum -y install ntp
psh compute yum -y install kernel
psh compute yum -y install lmod-ohpc

# ----------------------------------------------
# Customize system configuration (Section 4.6.2)
# ----------------------------------------------
echo "/home *(rw,no_subtree_check,fsid=10,no_root_squash)" >> /etc/exports
echo "/opt/ohpc/pub *(ro,no_subtree_check,fsid=11)" >> /etc/exports
exportfs -a
systemctl restart nfs-server
systemctl enable nfs-server
psh compute echo "\""${sms_ip}:/home /home nfs nfsvers=3,nodev,nosuid 0 0"\"" \>\> /etc/fstab
psh compute echo "\""${sms_ip}:/opt/ohpc/pub /opt/ohpc/pub nfs nfsvers=3,nodev 0 0"\"" \>\> /etc/fstab
psh compute systemctl restart nfs
psh compute mount /home
psh compute mkdir -p /opt/ohpc/pub
psh compute mount /opt/ohpc/pub
psh compute echo "\""server ${sms_ip}"\"" \>\> /etc/ntp.conf
psh compute systemctl enable ntpd
psh compute systemctl start ntpd

# Update basic slurm configuration if additional computes defined
# This is performed on the SMS, nodes will pick it up config file is copied there later
if [ ${num_computes} -gt 4 ];then
   perl -pi -e "s/^NodeName=(\S+)/NodeName=${compute_prefix}[1-${num_computes}]/" /etc/slurm/slurm.conf
   perl -pi -e "s/^PartitionName=normal Nodes=(\S+)/PartitionName=normal Nodes=${compute_prefix}[1-${num_computes}]/" /etc/slurm/slurm.conf
fi

# Update memlock settings
perl -pi -e 's/# End of file/\* soft memlock unlimited\n$&/s' /etc/security/limits.conf
perl -pi -e 's/# End of file/\* hard memlock unlimited\n$&/s' /etc/security/limits.conf
psh compute perl -pi -e "'s/# End of file/\* soft memlock unlimited\n$&/s' /etc/security/limits.conf"
psh compute perl -pi -e "'s/# End of file/\* hard memlock unlimited\n$&/s' /etc/security/limits.conf"

# Enable slurm pam module
psh compute echo "\""account required pam_slurm.so"\"" \>\> /etc/pam.d/sshd

# Enable Optional packages

if [[ ${enable_lustre_client} -eq 1 ]];then
     # Install Lustre client on master
     yum -y install lustre-client-ohpc
     # Enable lustre on compute nodes
     psh compute yum -y install lustre-client-ohpc
     psh compute mkdir /mnt/lustre
     psh compute echo  \ 
     # Enable o2ib for Lustre
     echo "options lnet networks=o2ib(ib0)" >> /etc/modprobe.d/lustre.conf
     psh compute echo "\""options lnet networks=o2ib\(ib0\)"\"" \>\> /etc/modprobe.d/lustre.conf
     # mount Lustre client on master and compute
     mkdir /mnt/lustre
     mount -t lustre -o localflock ${mgs_fs_name} /mnt/lustre
     psh compute mount /mnt/lustre
fi

if [[ ${enable_nagios} -eq 1 ]];then
     # Install Nagios on master and compute
     yum -y install ohpc-nagios
     psh compute yum -y install nagios-plugins-all-ohpc nrpe-ohpc
     psh compute systemctl enable nrpe
     psh compute perl -pi -e "\""s/^allowed_hosts=/\# allowed_hosts=/"\"" /etc/nagios/nrpe.cfg
     psh compute echo "\""nrpe 5666/tcp \# NRPE"\""         \>\> /etc/services
     psh compute echo "\""nrpe : ${sms_ip}  : ALLOW"\"" \>\> /etc/hosts.allow
     psh compute echo "\""nrpe : ALL : DENY"\""         \>\> /etc/hosts.allow
     psh compute "/usr/sbin/useradd -c "\""NRPE user for the NRPE service"\"" -d /var/run/nrpe -r -g nrpe -s /sbin/nologin nrpe"
     psh compute /usr/sbin/groupadd -r nrpe
     mv /etc/nagios/conf.d/services.cfg.example /etc/nagios/conf.d/services.cfg
     mv /etc/nagios/conf.d/hosts.cfg.example /etc/nagios/conf.d/hosts.cfg
     for ((i=0; i<$num_computes; i++)) ; do
        perl -pi -e "s/HOSTNAME$(($i+1))/${c_name[$i]}/ || s/HOST$(($i+1))_IP/${c_ip[$i]}/" \
        /etc/nagios/conf.d/hosts.cfg
     done
     perl -pi -e "s/ \/bin\/mail/ \/usr\/bin\/mailx/g" /etc/nagios/objects/commands.cfg
     perl -pi -e "s/nagios\@localhost/root\@${sms_name}/" /etc/nagios/objects/contacts.cfg
     psh compute "echo command[check_ssh]=/usr/lib64/nagios/plugins/check_ssh localhost >> /etc/nagios/nrpe.cfg"
     htpasswd -bc /etc/nagios/passwd nagiosadmin ${nagios_web_password}
     chkconfig nagios on
     systemctl start nagios
     chmod u+s `which ping`
     psh compute systemctl start  nrpe
fi

if [[ ${enable_ganglia} -eq 1 ]];then
     # Install Ganglia on master
     yum -y install ohpc-ganglia
     psh compute yum -y install ganglia-gmond-ohpc
     cp /opt/ohpc/pub/examples/ganglia/gmond.conf /etc/ganglia/gmond.conf
     perl -pi -e "s/<sms>/${sms_name}/" /etc/ganglia/gmond.conf
     xdcp compute /etc/ganglia/gmond.conf /etc/ganglia/gmond.conf
     echo "gridname MySite" >> /etc/ganglia/gmetad.conf
     systemctl enable gmond
     systemctl enable gmetad
     systemctl start gmond
     systemctl start gmetad
     psh compute systemctl enable gmond
     psh compute systemctl start gmond
     systemctl try-restart httpd
fi

if [[ ${enable_clustershell} -eq 1 ]];then
     # Install clustershell
     yum -y install clustershell-ohpc
     cd /etc/clustershell/groups.d
     mv local.cfg local.cfg.orig
     echo "adm: ${sms_name}" > local.cfg
     echo "compute: ${compute_prefix}[1-${num_computes}]" >> local.cfg
     echo "all: @adm,@compute" >> local.cfg
fi

if [[ ${enable_mrsh} -eq 1 ]];then
     # Install mrsh
     yum -y install mrsh-ohpc mrsh-rsh-compat-ohpc
     psh compute yum -y install mrsh-ohpc mrsh-rsh-compat-ohpc mrsh-server-ohpc
     echo "mshell          21212/tcp                  # mrshd" >> /etc/services
     echo "mlogin            541/tcp                  # mrlogind" >> /etc/services
     psh compute systemctl enable xinetd
     psh compute systemctl start  xinetd
fi

if [[ ${enable_genders} -eq 1 ]];then
     # Install genders
     yum -y install genders-ohpc
     echo -e "${sms_name}\tsms" > /etc/genders
     for ((i=0; i<$num_computes; i++)) ; do
        echo -e "${c_name[$i]}\tcompute,bmc=${c_bmc[$i]}"
     done >> /etc/genders
fi

# Optionally, enable conman and configure
if [[ ${enable_ipmisol} -eq 1 ]];then
     yum -y install conman-ohpc
     for ((i=0; i<$num_computes; i++)) ; do
        echo -n 'CONSOLE name="'${c_name[$i]}'" dev="ipmi:'${c_bmc[$i]}'" '
        echo 'ipmiopts="'U:${bmc_username},P:${IPMI_PASSWORD:-undefined},W:solpayloadsize'"'
     done >> /etc/conman.conf
     systemctl enable conman
     systemctl start conman
fi

# Optionally, enable nhc and configure
yum -y install nhc-ohpc
psh compute yum -y install nhc-ohpc

echo "HealthCheckProgram=/usr/sbin/nhc" >> /etc/slurm/slurm.conf
echo "HealthCheckInterval=300" >> /etc/slurm/slurm.conf  # execute every five minutes

# ---------------------------------------
# Install Development Tools (Section 5.1)
# ---------------------------------------
yum -y install ohpc-autotools
yum -y install EasyBuild-ohpc
yum -y install hwloc-ohpc
yum -y install spack-ohpc
yum -y install valgrind-ohpc

# -------------------------------
# Install Compilers (Section 5.2)
# -------------------------------
yum -y install gnu8-compilers-ohpc

# ----------------------
# Install llvm Compilers
# ----------------------
yum -y install llvm5-compilers-ohpc

# --------------------------------
# Install MPI Stacks (Section 5.3)
# --------------------------------
if [[ ${enable_mpi_defaults} -eq 1 && ${enable_pmix} -eq 0 ]];then
     yum -y install openmpi3-gnu8-ohpc mpich-gnu8-ohpc
elif [[ ${enable_mpi_defaults} -eq 1 && ${enable_pmix} -eq 1 ]];then
     yum -y install openmpi3-pmix-slurm-gnu8-ohpc mpich-gnu8-ohpc
fi

if [[ ${enable_ib} -eq 1 ]];then
     yum -y install mvapich2-gnu8-ohpc
fi
if [[ ${enable_opa} -eq 1 ]];then
     yum -y install mvapich2-psm2-gnu8-ohpc
fi

# ---------------------------------------
# Install Performance Tools (Section 5.4)
# ---------------------------------------
yum -y install ohpc-gnu8-perf-tools
yum -y install lmod-defaults-gnu8-openmpi3-ohpc

# ---------------------------------------------------
# Install 3rd Party Libraries and Tools (Section 5.6)
# ---------------------------------------------------
yum -y install ohpc-gnu8-serial-libs
yum -y install ohpc-gnu8-io-libs
yum -y install ohpc-gnu8-python-libs
yum -y install ohpc-gnu8-runtimes
if [[ ${enable_mpi_defaults} -eq 1 ]];then
     yum -y install ohpc-gnu8-mpich-parallel-libs
     yum -y install ohpc-gnu8-openmpi3-parallel-libs
fi
if [[ ${enable_ib} -eq 1 ]];then
     yum -y install ohpc-gnu8-mvapich2-parallel-libs
fi
if [[ ${enable_opa} -eq 1 ]];then
     yum -y install ohpc-gnu8-mvapich2-parallel-libs
fi

# -----------------------------------------------------------------------------------
# Install Optional Development Tools for use with Intel Parallel Studio (Section 5.7)
# -----------------------------------------------------------------------------------
if [[ ${enable_intel_packages} -eq 1 ]];then
     yum -y install intel-compilers-devel-ohpc
     yum -y install intel-mpi-devel-ohpc
     if [[ ${enable_opa} -eq 1 ]];then
          yum -y install mvapich2-psm2-intel-ohpc
     fi
     yum -y install ohpc-intel-serial-libs
     yum -y install ohpc-intel-io-libs
     yum -y install ohpc-intel-perf-tools
     yum -y install ohpc-intel-python-libs
     yum -y install ohpc-intel-runtimes
     yum -y install ohpc-intel-mpich-parallel-libs
     yum -y install ohpc-intel-mvapich2-parallel-libs
     yum -y install ohpc-intel-openmpi3-parallel-libs
     yum -y install ohpc-intel-impi-parallel-libs
fi

# ------------------------------------
# Resource Manager Startup (Section 6)
# ------------------------------------
xdcp compute /etc/slurm/slurm.conf /etc/slurm/slurm.conf
xdcp compute /etc/munge/munge.key /etc/munge/munge.key
systemctl enable munge
systemctl enable slurmctld
systemctl start munge
systemctl start slurmctld
psh compute systemctl enable munge
psh compute systemctl enable slurmd
psh compute systemctl start munge
psh compute systemctl start slurmd
useradd -m test
echo "MERGE:" > syncusers
echo "/etc/passwd -> /etc/passwd" >> syncusers
echo "/etc/group -> /etc/group"       >> syncusers
echo "/etc/shadow -> /etc/shadow" >> syncusers
xdcp compute -F syncusers
